/***********************************************************************
*
*  FILE        : Smart_Configurator_Example_USB.c
*  DATE        : 2022-04-22
*  DESCRIPTION : Main Program
*
*  NOTE:THIS IS A TYPICAL EXAMPLE.
*
***********************************************************************/
#include "r_smc_entry.h"
#include "r_usb_basic_if.h"
#include "r_usb_pcdc_if.h"
#include "r_usb_pcdc_descriptor.h"
#include <stdio.h>
#include <string.h>


void R_USB_PinSet_USB0_PERI();			/* Initialize USB0_VBUS pin */
static uint8_t g_buf[1];				/* Variable to store input character from PC terminal */
volatile unsigned int flag_start = 1;	/* Flag to print start message */
volatile uint16_t interval_level = 1;	/* Variable to change CMT0 interval */
static char print_str[120];				/* String to print message at PC terminal */
volatile uint32_t slength;				/* String length */
extern uint8_t g_apl_device[];
extern uint8_t g_apl_configuration[];
extern uint8_t *g_apl_string_table[];
const static usb_descriptor_t usb_descriptor =
{
	g_apl_device,			/* Pointer to the device descriptor */
	g_apl_configuration,	/* Pointer to the configuration descriptor for Full-speed */
	USB_NULL,				/* Pointer to the configuration descriptor for Hi-speed */
	USB_NULL,				/* Pointer to the qualifier descriptor */
	g_apl_string_table		/* Pointer to the string descriptor table */
};

void setup(void);
void loop(void);
void main(void);

void main(void)
{
	setup();
	for(;;)
	{
		loop();
	}
}

usb_ctrl_t ctrl;
usb_cfg_t cfg;

void setup(void)
{
	/* Start CMT0 counter operation */
	R_Config_CMT0_Start();

	R_USB_PinSet_USB0_PERI();		/* USB MCU pin setting */

	ctrl.module = USB_IP0;			/* USB0 module */
	ctrl.type = USB_PCDC;			/* Peripheral Communication Device Class*/
	cfg.usb_speed = USB_FS;			/* USB_HS/USB_FS */
	cfg.usb_mode = USB_PERI;
	cfg.p_usb_reg = (usb_descriptor_t *)&usb_descriptor;
	R_USB_Open(&ctrl, &cfg);		/* Initializes the USB module */

	while(USB_STS_CONFIGURED != R_USB_GetEvent(&ctrl))
	{
		nop();
	}

	/* Print start message (until the 1st input from PC host is received) */
	sprintf(print_str, "Press space bar twice(?) to start ...\r ");
	slength = strlen(print_str);
	/* Call the non blocking USB WRITE API to print the message */
	R_USB_Write(&ctrl, (uint8_t *)print_str, slength);

	/* Read the input from PC terminal */
	/* Call the non blocking USB READ API to get an input */
	R_USB_Read(&ctrl, g_buf, 1);
}

void loop(void)
{
	/* Loop back between PC Terminal and USB MCU */
	switch (R_USB_GetEvent(&ctrl))
	{
	case USB_STS_WRITE_COMPLETE :
		if(1 == flag_start)
		{
			/* Wait for a while */
			R_BSP_SoftwareDelay(1000, BSP_DELAY_MILLISECS);
			/* Print start message until the 1st input from PC host is received */
			sprintf(print_str, "Press space bar twice(?) to start ...\r ");
			slength = strlen(print_str);
			/* Call the non blocking USB WRITE API to print the message */
			R_USB_Write(&ctrl, (uint8_t *)print_str, slength);
		}
		break;

	case USB_STS_READ_COMPLETE :
		/* Clear start message flag */
		flag_start = 0;
		/* Instruction to slow down blinking rate is received */
		if (('a' == *g_buf)|| ('A' == *g_buf))
		{
		    /* Get new blink interval level. Maximum level is 9. */
			if (interval_level < 10)
			{/* Notify the character received and inform next action */
				sprintf(print_str, "\r\n Character 'a' or 'A' is received. LED2 will blink at slower rate.\r\n");
				interval_level++;
			}
			else
			{
				sprintf(print_str, "\r\n This is minimum. Please press 'b' or 'B' to blink faster. \r\n");
			}
		}
		else if (('b' == *g_buf)|| ('B' == *g_buf))
		{
			/* Get new blink interval level. Minimum level is 1 */
			if (interval_level > 1)
			{
				/* Instruction to increase blinking rate is received */
				sprintf(print_str, "\r\n Character 'b' or 'B' is received. LED2 will blink at faster rate.\r\n");
				interval_level--;
			}
			else
			{
				sprintf(print_str, "\r\n This is maximum. Please press 'a' or 'A' to blink slower. \r\n");
			}
		}
		else
		{
			sprintf(print_str, "\r\n Press keyboard to control the blinking rate of LED2 (not case sensitive)\n\r\n a ----> Slower\n\r\n b ----> Faster\n\r\n");
		}
		/* Print message at PC terminal */
		slength = strlen(print_str);
		/* Call the non blocking USB WRITE API to print the message */
		R_USB_Write(&ctrl, (uint8_t *)print_str, slength);
        /* ↑細かいことを言うとビジーチェックが無いですが、、、(前の送信完了前に送信するリスクがありますが、、、) */

		/* Change blinking rate */
		CMT0.CMCOR = (uint16_t)(5000 * interval_level);

		/* Read the input from PC terminal */
		/* Call the non blocking USB READ API to get an input */
		R_USB_Read(&ctrl, g_buf, 1);
		break;

	default :
		break;
	}
}
